#include "mainwindow.h"

#include <qmenu.h>
#include <qframe.h>
#include <qaction.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qtextedit.h>
#include <qstatusbar.h>
#include <qdockwidget.h>

MainWindow::MainWindow( QWidget *parent, Qt::WFlags flags )
        : QMainWindow( parent, flags )
{
    setObjectName( "MainWindow" );
    QMenu *dockMenu = new QMenu( tr( "&Docks" ), this );

    m_topDockBarAction = dockMenu->addAction( tr( "&Top DockBar" ) );
    m_topDockBarAction->setCheckable( true );
    m_topDockBarAction->setChecked( true );
    m_bottomDockBarAction = dockMenu->addAction( tr( "&Bottom DockBar" ) );
    m_bottomDockBarAction->setCheckable( true );
    m_bottomDockBarAction->setChecked( true );
    m_leftDockBarAction = dockMenu->addAction( tr( "&Left DockBar" ) );
    m_leftDockBarAction->setCheckable( true );
    m_leftDockBarAction->setChecked( true );
    m_rightDockBarAction = dockMenu->addAction( tr( "&Right DockBar" ) );
    m_rightDockBarAction->setCheckable( true );
    m_rightDockBarAction->setChecked( true );

    menuBar() ->addMenu( dockMenu );

    QMenu *cornerMenu = new QMenu( tr( "&Corners" ), this );
    m_verticalCorner = cornerMenu->addAction( tr( "&Vertical Corners" ) );
    m_verticalCorner->setCheckable( true );
    m_horizontalCorner = cornerMenu->addAction( tr( "&Horizontal Corners" ) );
    m_horizontalCorner->setCheckable( true );

    QActionGroup *cornerGroup = new QActionGroup( this );
    cornerGroup->addAction(m_verticalCorner);
    cornerGroup->addAction(m_horizontalCorner);

    menuBar() ->addMenu( cornerMenu );

    connect( dockMenu, SIGNAL( triggered( QAction * ) ),
             this, SLOT( dockBarEnabled( QAction * ) ) );
    connect( cornerGroup, SIGNAL( triggered( QAction * ) ),
             this, SLOT( setCornerArea( QAction * ) ) );

    QToolBar *testBar = new QToolBar( this );
    testBar->addAction( tr( "Tool Bar" ) );
    addToolBar( testBar );

    createDocks();
    setDockBarEnabled( Qt::TopDockWidgetArea, true );
    setDockBarEnabled( Qt::BottomDockWidgetArea, true );
    setDockBarEnabled( Qt::LeftDockWidgetArea, true );
    setDockBarEnabled( Qt::RightDockWidgetArea, true );

    QTextEdit *center = new QTextEdit( this );
    center->setReadOnly( true );
    center->setHtml( tr( "Center Widget" ) );
    center->setMinimumSize( 0, 0 );
    setCentralWidget( center );

    statusBar() ->showMessage( tr( "Status Bar" ) );
}

MainWindow::~ MainWindow( )
{}

void MainWindow::dockBarEnabled( QAction *action )
{
    if ( action == m_topDockBarAction )
        setDockBarEnabled( Qt::TopDockWidgetArea, action->isChecked() );
    else if ( action == m_bottomDockBarAction )
        setDockBarEnabled( Qt::BottomDockWidgetArea, action->isChecked() );
    else if ( action == m_leftDockBarAction )
        setDockBarEnabled( Qt::LeftDockWidgetArea, action->isChecked() );
    else if ( action == m_rightDockBarAction )
        setDockBarEnabled( Qt::RightDockWidgetArea, action->isChecked() );
}

void MainWindow::setCornerArea( QAction *action )
{
    if ( action == m_verticalCorner )
    {
        setCorner( Qt::TopLeftCorner, Qt::LeftDockWidgetArea );
        setCorner( Qt::TopRightCorner, Qt::RightDockWidgetArea );
        setCorner( Qt::BottomLeftCorner, Qt::LeftDockWidgetArea );
        setCorner( Qt::BottomRightCorner, Qt::RightDockWidgetArea );
    }
    else
    {
        setCorner( Qt::TopLeftCorner, Qt::TopDockWidgetArea );
        setCorner( Qt::TopRightCorner, Qt::TopDockWidgetArea );
        setCorner( Qt::BottomLeftCorner, Qt::BottomDockWidgetArea );
        setCorner( Qt::BottomRightCorner, Qt::BottomDockWidgetArea );
    }
    layout()->invalidate();
    layout()->activate();
}

void MainWindow::createDocks()
{
    QDockWidget * foobar = new QDockWidget( "FooBar", this );
    QDockWidget * barfoo = new QDockWidget( "BarFoo", this );
    QDockWidget * foobaz = new QDockWidget( "FooBaz", this );
    QDockWidget * bazbarfoo = new QDockWidget( "BazBarFoo", this );
    QDockWidget * foobarbaz = new QDockWidget( "FooBarBaz", this );

    foobar->setWindowIcon( QIcon(":/images/red.png") );
    barfoo->setWindowIcon( QIcon(":/images/green.png") );
    foobaz->setWindowIcon( QIcon(":/images/blue.png") );
    bazbarfoo->setWindowIcon( QIcon(":/images/yellow.png") );
    foobarbaz->setWindowIcon( QIcon(":/images/black.png") );

    foobar->setMinimumSize( 200, 100 );
    barfoo->setMinimumSize( 200, 100 );
    foobaz->setMinimumSize( 200, 100 );
    bazbarfoo->setMinimumSize( 200, 100 );
    foobarbaz->setMinimumSize( 200, 100 );

    QFrame *foobarFrame = new QFrame( foobar );
    foobarFrame->setFrameStyle( QFrame::Box | QFrame::Sunken );
    foobarFrame->setMinimumSize( 100, 75 );

    QFrame *barfooFrame = new QFrame( barfoo );
    barfooFrame->setFrameStyle( QFrame::Box | QFrame::Sunken );
    barfooFrame->setMinimumSize( 150, 75 );

    QFrame *foobazFrame = new QFrame( foobaz );
    foobazFrame->setFrameStyle( QFrame::Box | QFrame::Sunken );
    foobazFrame->setMinimumSize( 150, 75 );

    QFrame *bazbarfooFrame = new QFrame( bazbarfoo );
    bazbarfooFrame->setFrameStyle( QFrame::Box | QFrame::Sunken );
    bazbarfooFrame->setMinimumSize( 150, 75 );

    QFrame *foobarbazFrame = new QFrame( foobarbaz );
    foobarbazFrame->setFrameStyle( QFrame::Box | QFrame::Sunken );
    foobarbazFrame->setMinimumSize( 150, 75 );

    foobar->setWidget( foobarFrame );
    barfoo->setWidget( barfooFrame );
    foobaz->setWidget( foobazFrame );
    bazbarfoo->setWidget( bazbarfooFrame );
    foobarbaz->setWidget( foobarbazFrame );

    addDockWidget( Qt::TopDockWidgetArea, foobar );
    addDockWidget( Qt::BottomDockWidgetArea, barfoo );
    addDockWidget( Qt::LeftDockWidgetArea, foobaz );
    addDockWidget( Qt::LeftDockWidgetArea, bazbarfoo );
    addDockWidget( Qt::RightDockWidgetArea, foobarbaz );
}
